@@ -45,6 +45,7 @@ struct _FileNodeData {
 	gchar *uri_suffix;
 	GArray *properties;
 	guint shallow   : 1;
+	guint unowned : 1;
 	guint file_type : 4;
 };
 
@@ -66,6 +67,14 @@ G_DEFINE_TYPE (TrackerFileSystem, tracker_file_system, G_TYPE_OBJECT)
  *     comparisons on them.
  *   - Stores data for the GFile lifetime, so it may be used as cache store
  *     as long as some file is needed.
+ *
+ * The TrackerFileSystem holds a reference on each GFile. There are two cases
+ * when we want to force a cached GFile to be freed: when it no longer exists
+ * on disk, and once crawling a directory has completed and we only need to
+ * remember the directories. Objects may persist in the cache even after
+ * tracker_file_system_forget_files() is called to delete them if there are
+ * references held on them elsewhere, and they will stay until all references
+ * are dropped.
  */
 
 
@@ -82,7 +91,9 @@ file_node_data_free (FileNodeData *data,
 			                     node);
 		}
 
-		g_object_unref (data->file);
+		if (!data->unowned) {
+			g_object_unref (data->file);
+		}
 	}
 
 	data->file = NULL;
@@ -820,13 +831,13 @@ typedef struct {
 	TrackerFileSystem *file_system;
 	GList *list;
 	GFileType file_type;
-} DeleteFilesData;
+} ForgetFilesData;
 
 static gboolean
 append_deleted_files (GNode    *node,
 		      gpointer  user_data)
 {
-	DeleteFilesData *data;
+	ForgetFilesData *data;
 	FileNodeData *node_data;
 
 	data = user_data;
@@ -834,18 +845,31 @@ append_deleted_files (GNode    *node,
 
 	if (data->file_type == G_FILE_TYPE_UNKNOWN ||
 	    node_data->file_type == data->file_type) {
-		data->list = g_list_prepend (data->list, node_data->file);
+		data->list = g_list_prepend (data->list, node_data);
 	}
 
 	return FALSE;
 }
 
+static void
+forget_file (FileNodeData *node_data)
+{
+	if (!node_data->unowned) {
+		node_data->unowned = TRUE;
+
+		/* Weak reference handler will remove the file from the tree and
+		 * clean up node_data if this is the final reference.
+		 */
+		g_object_unref (node_data->file);
+	}
+}
+
 void
-tracker_file_system_delete_files (TrackerFileSystem *file_system,
+tracker_file_system_forget_files (TrackerFileSystem *file_system,
 				  GFile             *root,
 				  GFileType          file_type)
 {
-	DeleteFilesData data = { file_system, NULL, file_type };
+	ForgetFilesData data = { file_system, NULL, file_type };
 	GNode *node;
 
 	g_return_if_fail (TRACKER_IS_FILE_SYSTEM (file_system));
@@ -864,6 +888,6 @@ tracker_file_system_delete_files (TrackerFileSystem *file_system,
 	                 -1, append_deleted_files,
 	                 &data);
 
-	g_list_foreach (data.list, (GFunc) g_object_unref, NULL);
+	g_list_foreach (data.list, (GFunc) forget_file, NULL);
 	g_list_free (data.list);
 }
