@@ -25,13 +25,17 @@
 #include <libtracker-miner/tracker-crawler.h>
 #include <libtracker-miner/tracker-storage.h>
 #include <libtracker-miner/tracker-thumbnailer.h>
-#include <libtracker-miner/tracker-albumart.h>
+#include <libtracker-miner/tracker-media-art.h>
 #include <libtracker-miner/tracker-network-provider.h>
 #include <libtracker-miner/tracker-password-provider.h>
 #include <libtracker-miner/tracker-miner-object.h>
 #include <libtracker-miner/tracker-miner-fs.h>
 #include <libtracker-miner/tracker-miner-web.h>
 #include <libtracker-miner/tracker-miner-manager.h>
+#include <libtracker-miner/tracker-miner-enums.h>
+#include <libtracker-miner/tracker-miner-enum-types.h>
+#include <libtracker-miner/tracker-indexing-tree.h>
+
 
 #undef __LIBTRACKER_MINER_H_INSIDE__
 
